@@ -5,7 +5,7 @@
 ;; Author: Jimmy Aguilar Mena
 ;; URL: https://github.com/Ergus/gtags-mode
 ;; Keywords: xref, project, imenu, gtags, global
-;; Version: 1.9.1
+;; Version: 1.9.2
 ;; Package-Requires: ((emacs "28"))
 
 ;; This program is free software: you can redistribute it and/or modify
@@ -140,6 +140,8 @@ warnings or errors."
 	 (connection-local-set-profiles criteria symvars))))
     (hack-connection-local-variables-apply criteria)))
 
+(defvar gtags-mode--pr-async nil)
+
 ;; Async functions
 (defun gtags-mode--exec-async-sentinel (process event)
   "Sentinel to run when PROCESS emits EVENT.
@@ -163,6 +165,7 @@ This is the sentinel set in `gtags-mode--exec-async'."
       (gtags-mode--message 1 "Global async error output:\n%s"
 			   (string-trim
 			    (buffer-substring-no-properties (point-min) (point-max))))))
+  (setq gtags-mode--pr-async nil)
   (gtags-mode--message 2 "Async %s: result: %s elapsed: %.06f"
 		       (process-get process :command)
 		       (string-trim event)
@@ -177,17 +180,22 @@ Returns the process object."
   (if-let* ((cmd (buffer-local-value cmd (current-buffer)))
 	    (command (append `(,cmd) (string-split gtags-mode-update-args) args))
 	    (start-time (current-time))
+	    ((if (not gtags-mode--pr-async)
+		t
+	      (gtags-mode--message 1 "Cannot run command: There is a gtags or global process already running.")
+	      nil))
 	    (pr (make-process :name (format "%s-async" cmd)
 			      :buffer (generate-new-buffer " *temp*" t)
 			      :command command
 			      :sentinel #'gtags-mode--exec-async-sentinel
 			      :file-handler t)))
       (progn
+	(gtags-mode--message 2 "Starting async process: %s" command)
 	;; In future not needed with `remote-commands'.
 	(set-process-plist pr (list :parent-buffer (current-buffer)
 				    :command command
 				    :start-time start-time))
-	pr)
+	(setq gtags-mode--pr-async pr))
     (gtags-mode--message 1 "Can't start async %s subprocess" cmd)
     nil))
 
@@ -313,11 +321,14 @@ name, code, file, line."
 This iterates over the buffers and tries to reset
 `gtags-mode--plist' when it is nil."
   (dolist (buff (buffer-list))
-    (unless (buffer-local-value 'gtags-mode--plist buff)
-      (with-current-buffer buff
-	(gtags-mode--set-connection-locals)
-	(kill-local-variable 'gtags-mode--plist) ;; kill the local to reset it
-	(gtags-mode--local-plist default-directory)))))
+    (gtags-mode--message 2 "Updating all buffers")
+    (with-current-buffer buff
+      (when buffer-file-name
+	(if (buffer-local-value 'gtags-mode--plist buff)
+	    (setq-local gtags-mode--list-cache-plist nil) ;; cleanup the cache 
+	  (gtags-mode--set-connection-locals)
+	  (kill-local-variable 'gtags-mode--plist) ;; kill the local to reset it
+	  (gtags-mode--local-plist default-directory))))))
 
 ;; Interactive commands ==============================================
 
@@ -332,9 +343,10 @@ This iterates over the buffers and tries to reset
 (defun gtags-mode-update ()
   "Update GLOBAL project database."
   (interactive)
-  (when (gtags-mode--local-plist default-directory)
-    (let ((default-directory (plist-get gtags-mode--plist :gtagsroot)))
-      (gtags-mode--exec-async 'gtags-mode--global "--update"))))
+  (when-let*  (((gtags-mode--local-plist default-directory))
+	       (default-directory (plist-get gtags-mode--plist :gtagsroot))
+	       (pr (gtags-mode--exec-async 'gtags-mode--global "--update")))
+    (process-put pr :extra-sentinel #'gtags-mode--update-buffers-plist)))
 
 ;; Hooks =============================================================
 (defun gtags-mode--after-save-hook ()
